--- /usr/local/share/xemacs-21.5-b32/lisp/menubar.el	2013-01-01 22:36:57.000000000 -0500
+++ /usr/src/xemacs-21.5.32/lisp/menubar.el	2013-01-03 13:54:54.000000000 -0500
@@ -36,6 +36,12 @@
   "Input from the menus."
   :group 'environment)
 
+(defcustom menu-developer-p (not (null xemacs-betaname))	;Defaults to t for beta XEmacs
+  "Perform Menu Debugging Checks." 
+  :tag "Menu Developer P"
+  :group 'debug
+  :type '(boolean))
+
 (defvar default-menubar nil)
 
 ;; this function is considered "part of the lexicon" by many,
@@ -209,6 +215,17 @@
                (cons result parent))))))
     (find-menu-item-1 menubar item-path-list)))
 
+(defmacro menu-developer-included-subfun (command-name)
+  ;; This macro is used as a default :included value for
+  ;; menu items with a symbol callback. If menu-developer-p,
+  ;; check if a command, and if so log a warning and return nil.
+  ;; Suggested by Stephen Turnbull
+  `(if (commandp ',command-name)
+      t
+    (when menu-developer-p
+	(warn (concat "Menu callbacks must be commands: " (symbol-name ',command-name))))
+    nil))
+
 (defun add-menu-item-1 (leaf-p menu-path new-item before in-menu)
   ;; This code looks like it could be cleaned up some more
   ;; Do we really need 6 calls to find-menu-item?
@@ -227,1 +244,1 @@
 		      (menu
		       (find-menu-item (cdr menu) (list item-name)))
 		      (t
-		       (find-menu-item menubar (list item-name)))
-		      )))
+		       (find-menu-item menubar (list item-name))))))
+    
+    ;; If length 3 or 4, and lacking any keywords
+    ;; the new-item is in deprecated form:
+    ;; [Name call-back activep suffix]
+    ;; This prevents us from adding any other tags
+    ;; (such as :included). Change to tag form for
+    ;; actual menu insertion.
+    (when (and (memq (length new-item) '(3 4))	
+	       (not (keywordp (aref new-item 2))))
+      (setq new-item `[,@(subseq new-item 0 2)
+		       :active
+		       ,(aref new-item 2)
+		       ,@(when (eq (length new-item) 4)
+			   `(:suffix
+			     ,(aref new-item 3)))]))
+
+    ;; Add default :included to check for non-command symbol callbacks at runtime
+    (when (and (vectorp new-item)		; If the new item has a symbol callback, and no :included.
+	       (symbolp (aref new-item 1))
+	       (not (find :included new-item)))
+      (setq new-item `[,@new-item		; Insert a default :included. See (`menu-developer-subfun')
+		       :included
+		       (menu-developer-included-subfun ,(aref new-item 1))]))
+		      
     (unless menubar
       (error "`current-menubar' is nil: can't add menus to it."))
     (unless menu
